import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { ExerciseService } from 'app/exercise/service/exercise.service';
import { ArtemisExerciseInfo } from 'app/shared/model/artemis-exercise-info.model';
import { AlertService } from 'app/core/util/alert.service';
import { Person } from 'app/shared/model/person.model';
import { GitlabService } from 'app/shared/gitlab/gitlab.service';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { GitlabPathSelectorComponent } from 'app/shared/gitlab/gitlab-path-selector/gitlab-path-selector.component';
import { GitlabGroup } from 'app/shared/model/gitlab-group.model';
import { COMMA, ENTER, TAB } from '@angular/cdk/keycodes';
import { MatChipInputEvent } from '@angular/material/chips';

@Component({
  selector: 'jhi-import',
  templateUrl: './exercise-import.component.html',
  styleUrls: ['./exercise-import.component.scss'],
})
export class ExerciseImportComponent implements OnInit {
  exerciseToken: string | undefined;
  exerciseInfo: ArtemisExerciseInfo | undefined;

  languages = ['english', 'german'];
  difficulties = ['simple', 'medium', 'advanced'];

  newCreator = { name: '', email: '', affiliation: '' } as Person;
  newPublisher = { name: '', email: '', affiliation: '' } as Person;
  isPublisherSameAsCreator = true;

  selectedGitlabGroup: GitlabGroup | undefined;
  isLoading = false;
  isImporting = false;

  separatorKeysCodes = [ENTER, COMMA, TAB];

  constructor(
    private route: ActivatedRoute,
    private exerciseService: ExerciseService,
    private jhiAlertService: AlertService,
    private gitlabService: GitlabService,
    private modalService: NgbModal
  ) {}

  ngOnInit(): void {
    this.isLoading = true;
    this.exerciseToken = this.route.snapshot.params['token'];
    if (this.exerciseToken) {
      this.exerciseService.importExerciseInfoFromArtemis(this.exerciseToken).subscribe(
        data => {
          this.exerciseInfo = data;
          this.exerciseInfo.programmingLanguage = data['programmingLanguage'];
          this.exerciseInfo.programmingLanguage = this.exerciseInfo.programmingLanguage.map(programmingLanguage =>
            this.toTitleCase(programmingLanguage)
          );
          this.exerciseInfo.creator = data['creator'];
          if (this.exerciseInfo.language === undefined || this.exerciseInfo.language === null || this.exerciseInfo.language.length === 0) {
            this.exerciseInfo.language = ['english']; // default value if none is set
          }
          if (this.exerciseInfo.identifier === undefined || this.exerciseInfo.identifier.localeCompare('')) {
            this.route.queryParams.subscribe(params => {
              this.exerciseInfo!.identifier = params['callback'];
            });
          }
          if (this.exerciseInfo.learningResourceType === undefined || this.exerciseInfo.learningResourceType.localeCompare('')) {
            this.exerciseInfo.learningResourceType = 'Artemis';
          }
          if (this.exerciseInfo.keyword === undefined || this.exerciseInfo.keyword.length < 1) {
            this.exerciseInfo.keyword = ['artemis'];
          }
        },
        error => {
          this.jhiAlertService.addAlert({ type: 'danger', message: 'Following error occurred: '.concat(error.error) });
        }
      );
    }
    this.isLoading = false;
  }

  /**
   * Actual import function submitting import form to server
   */
  import(): void {
    if (this.exerciseInfo!.difficulty !== null) {
      this.exerciseInfo!.difficulty = this.exerciseInfo!.difficulty?.toLowerCase();
    }
    if (this.isPublisherSameAsCreator) {
      if (this.exerciseInfo) this.exerciseInfo.publisher = this.exerciseInfo?.creator;
    }
    this.openGitlabPathSelectorModal().then(gitlabGroup => {
      if (gitlabGroup !== undefined) {
        this.isLoading = true;
        this.isImporting = true;
        this.exerciseService.submitExerciseInfoForImport(this.exerciseInfo!, this.exerciseToken!, gitlabGroup.id).subscribe(
          () => {
            window.location.href = this.route.snapshot.queryParams['callback'];
          },
          error => {
            this.jhiAlertService.addAlert({ type: 'danger', message: error.error.detail });
            window.scroll(0, 0);
            this.isLoading = false;
            this.isImporting = false;
          }
        );
      }
    });
    this.isLoading = false;
    this.isImporting = false;
  }

  /**
   * Utility function used to open the Gitlab selector modal
   */
  openGitlabPathSelectorModal(): Promise<GitlabGroup> {
    const modalRef = this.modalService.open(GitlabPathSelectorComponent, { size: 'xl', backdrop: 'static' });
    return modalRef.result.then(group => {
      this.selectedGitlabGroup = group;
      return group as GitlabGroup;
    });
  }

  /**
   * Validation function for button-groups.
   * At least one value must be set.
   * @return true if invalid and false if valid
   */
  hasValidationErrors(): boolean {
    if (
      this.exerciseInfo?.keyword === undefined ||
      this.exerciseInfo?.language === undefined ||
      this.exerciseInfo.creator === undefined ||
      !(this.exerciseInfo.keyword.length > 0) ||
      !(this.exerciseInfo.language.length > 0) ||
      !(this.exerciseInfo.creator.length > 0) ||
      (!this.isPublisherSameAsCreator && (this.exerciseInfo.publisher === undefined || !(this.exerciseInfo.publisher.length > 0)))
    ) {
      return true;
    }
    return false;
  }

  /**
   * Used to set the exercise's difficulty on
   * user's click
   * @param difficulty to set
   */
  setDifficulty(difficulty: string): void {
    this.exerciseInfo!.difficulty = difficulty;
  }

  /**
   * Used to add a language to the exercise
   * @param language to add
   */
  setLanguage(language: string): void {
    if (this.exerciseInfo!.language.includes(language)) {
      this.exerciseInfo!.language = this.exerciseInfo!.language.filter(lang => lang !== language);
    } else {
      this.exerciseInfo!.language.push(language);
    }
  }

  /**
   * Used to add a new person to a given list of persons
   * @param person to add
   * @param personList to add the given person to (ex. 'creators', 'publishers')
   */
  addPerson(person: Person, personList: string): void {
    if (this.validatePerson(person) && this.exerciseInfo) {
      if (!personList.localeCompare('creator')) {
        if (this.exerciseInfo.creator == null) {
          this.exerciseInfo.creator = [];
        }
        this.exerciseInfo.creator.push(JSON.parse(JSON.stringify(person)));
      } else if (!personList.localeCompare('publisher')) {
        if (this.exerciseInfo.publisher == null) {
          this.exerciseInfo.publisher = [];
        }
        this.exerciseInfo.publisher.push(JSON.parse(JSON.stringify(person)));
      }

      // needed in order to modify person object by reference
      person.name = '';
      person.email = '';
      person.affiliation = '';
    }
  }

  /**
   * Used to remove a person from a given list of persons
   * @param personToRemove to remove
   * @param personList to remove the given person from (ex. 'creators', 'publishers')
   */
  removePerson(personToRemove: Person, personList: string): void {
    if (!personList.localeCompare('creator')) {
      this.exerciseInfo!.creator = this.exerciseInfo!.creator.filter(person => person !== personToRemove);
    } else if (!personList.localeCompare('publisher')) {
      this.exerciseInfo!.publisher = this.exerciseInfo!.publisher.filter(person => person !== personToRemove);
    }
  }

  /**
   * Validates person by checking if all attributes are set
   *
   * @param person to check
   * @private
   */
  private validatePerson(person: Person): boolean {
    if (person.name === undefined || person.affiliation === undefined || person.email === undefined) {
      return false;
    }
    if (person.name === '' || person.affiliation === '' || person.email === '') {
      return false;
    }
    return true;
  }

  /**
   * Utility function used to turn words into title case
   * @param word the word to turn into title case
   */
  private toTitleCase(word: string): string {
    return word[0].toUpperCase() + word.substr(1).toLowerCase();
  }

  /**
   * Utility function used to check if the given difficulty is
   * the same as the imported exercise
   * @param difficulty to check
   */
  isDifficulty(difficulty: string): boolean {
    if (this.exerciseInfo!.difficulty !== undefined) {
      return this.exerciseInfo!.difficulty.toLowerCase() === difficulty.toLowerCase();
    } else {
      return false;
    }
  }

  /**
   * Utility function used to check if a given language is set
   * in the imported exercsie
   * @param language
   */
  isLanguage(language: string): boolean {
    return this.exerciseInfo!.language !== undefined && this.exerciseInfo!.language.includes(language);
  }

  /**
   * Used to validate language input
   */
  isLanguageSet(): boolean {
    return this.exerciseInfo!.language !== undefined && this.exerciseInfo!.language.length > 0;
  }

  /**
   * Checks if at least one person is set for the given list of persons
   */
  isPersonSet(personList: Person[]): boolean {
    return personList !== undefined && personList.length > 0;
  }

  onKeywordAdd(event: MatChipInputEvent): void {
    if (this.exerciseInfo!.keyword.filter(entry => entry === event.value).length === 0) {
      this.exerciseInfo!.keyword.push(event.value);
    }
    event.chipInput?.clear();
  }

  onKeywordRemove(keyword: string): void {
    this.exerciseInfo!.keyword = this.exerciseInfo!.keyword.filter(entry => entry !== keyword);
  }

  onProgrammingLanguageAdd(event: MatChipInputEvent): void {
    if (this.exerciseInfo!.programmingLanguage.filter(entry => entry === event.value).length === 0) {
      this.exerciseInfo!.programmingLanguage.push(event.value);
    }
    event.chipInput?.clear();
  }

  onProgrammingLanguageRemove(programmingLanguage: string): void {
    this.exerciseInfo!.programmingLanguage = this.exerciseInfo!.programmingLanguage.filter(entry => entry !== programmingLanguage);
  }
}
